@@ -30,13 +30,7 @@
 #include "jsapi-util.h"
 #include "jsapi-wrapper.h"
 
-#ifdef __GNUC__
-#define GNUC_ALWAYS_INLINE __attribute__((always_inline))
-#else
-#define GNUC_ALWAYS_INLINE
-#endif
-
-GNUC_ALWAYS_INLINE
+GJS_ALWAYS_INLINE
 static inline bool
 check_nullable(const char*& fchar,
                const char*& fmt_string)
@@ -53,7 +47,7 @@ check_nullable(const char*& fchar,
 
 /* This preserves the previous behaviour of gjs_parse_args(), but maybe we want
  * to use JS::ToBoolean instead? */
-GNUC_ALWAYS_INLINE
+GJS_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -72,7 +66,7 @@ assign(JSContext      *cx,
 
 /* This preserves the previous behaviour of gjs_parse_args(), but maybe we want
  * to box primitive types instead of throwing? */
-GNUC_ALWAYS_INLINE
+GJS_ALWAYS_INLINE
 static inline void
 assign(JSContext              *cx,
        char                    c,
@@ -91,34 +85,43 @@ assign(JSContext              *cx,
     ref.set(&value.toObject());
 }
 
-GNUC_ALWAYS_INLINE
+GJS_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
        bool            nullable,
        JS::HandleValue value,
-       char          **ref)
+       GjsAutoJSChar  *ref)
 {
-    if (nullable && (c == 's' || c == 'F') && value.isNull()) {
-        *ref = NULL;
+    if (c != 's')
+        throw g_strdup_printf("Wrong type for %c, got GjsAutoJSChar*", c);
+    if (nullable && value.isNull()) {
+        ref->reset(cx, nullptr);
         return;
     }
-
-    GjsAutoJSChar tmp_ref(cx);
-
-    if (c == 's') {
-        if (!gjs_string_to_utf8(cx, value, &tmp_ref))
-            throw g_strdup("Couldn't convert to string");
-    } else if (c == 'F') {
-        if (!gjs_string_to_filename(cx, value, &tmp_ref))
-            throw g_strdup("Couldn't convert to filename");
-    } else {
-        throw g_strdup_printf("Wrong type for %c, got char**", c);
-    }
-
-    *ref = tmp_ref.copy();
+    if (!gjs_string_to_utf8(cx, value, ref))
+        throw g_strdup("Couldn't convert to string");
 }
-GNUC_ALWAYS_INLINE
+
+GJS_ALWAYS_INLINE
+static inline void
+assign(JSContext      *cx,
+       char            c,
+       bool            nullable,
+       JS::HandleValue value,
+       GjsAutoChar    *ref)
+{
+    if (c != 'F')
+        throw g_strdup_printf("Wrong type for %c, got GjsAutoChar*", c);
+    if (nullable && value.isNull()) {
+        ref->release();
+        return;
+    }
+    if (!gjs_string_to_filename(cx, value, ref))
+        throw g_strdup("Couldn't convert to filename");
+}
+
+GJS_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -134,7 +137,7 @@ assign(JSContext      *cx,
         throw g_strdup("Couldn't convert to integer");
 }
 
-GNUC_ALWAYS_INLINE
+GJS_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -155,7 +158,7 @@ assign(JSContext      *cx,
     *ref = num;
 }
 
-GNUC_ALWAYS_INLINE
+GJS_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -171,7 +174,7 @@ assign(JSContext      *cx,
         throw g_strdup("Couldn't convert to 64-bit integer");
 }
 
-GNUC_ALWAYS_INLINE
+GJS_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -191,7 +194,7 @@ assign(JSContext      *cx,
  * prevent instantiation for any other types besides pointer-to-enum */
 template<typename T,
          typename std::enable_if<std::is_enum<T>::value, int>::type = 0>
-GNUC_ALWAYS_INLINE
+GJS_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -217,7 +220,7 @@ template<typename T,
 static inline void
 free_if_necessary(T param_ref) {}
 
-GNUC_ALWAYS_INLINE
+GJS_ALWAYS_INLINE
 static inline void
 free_if_necessary(JS::MutableHandleObject param_ref)
 {
@@ -227,13 +230,6 @@ free_if_necessary(JS::MutableHandleObject param_ref)
     param_ref.set(NULL);
 }
 
-GNUC_ALWAYS_INLINE
-static inline void
-free_if_necessary(char **param_ref)
-{
-    g_free(*param_ref);
-}
-
 template<typename T>
 static bool
 parse_call_args_helper(JSContext    *cx,
