@@ -13,7 +13,7 @@ describe('Coverage.expressionLinesForAST', function () {
             "x = (function() {\n" +
             "    return 10;\n" +
             "})();\n",
-            [1, 2, 3],
+            [2, 3],
         ],
 
         'finds lines inside functions': [
@@ -57,23 +57,23 @@ describe('Coverage.expressionLinesForAST', function () {
         ],
 
         'finds lines inside multiline function arguments': [
-            "function f(a, b, c) {\n" +
-            "}\n" +
-            "f(1,\n" +
-            "  2,\n" +
-            "  3);\n",
+            `function f(a, b, c) {
+            }
+            f(1,
+                2 + 3,
+                3 + 4);`,
             [1, 3, 4, 5],
         ],
 
         'finds lines inside function argument that is an object': [
-            "function f(o) {\n" +
-            "}\n" +
-            "let obj = {\n" +
-            "    Name: new f({ a: 1,\n" +
-            "                  b: 2,\n" +
-            "                  c: 3\n" +
-            "                })\n" +
-            "}\n",
+            `function f(o) {
+            }
+            let obj = {
+                Name: new f({ a: 1,
+                    b: 2 + 3,
+                    c: 3 + 4,
+                })
+            } `,
             [1, 3, 4, 5, 6],
         ],
 
@@ -154,10 +154,10 @@ describe('Coverage.expressionLinesForAST', function () {
         ],
 
         'finds lines for object property literals': [
-            "var a = {\n" +
-            "    Name: 'foo',\n" +
-            "    Ex: 'bar'\n" +
-            "};\n",
+            `var a = {
+                Name: 'foo' + 'bar',
+                Ex: 'bar' + 'foo',
+            }`,
             [1, 2, 3],
         ],
 
@@ -190,14 +190,24 @@ describe('Coverage.expressionLinesForAST', function () {
         ],
 
         'finds lines inside object-valued argument to throw statement': [
-            "function f() {\n" +
-            "    throw {\n" +
-            "        a: 1,\n" +
-            "        b: 2\n" +
-            "    }\n" +
-            "}\n",
+            `function f() {
+                throw {
+                    a: 1 + 2,
+                    b: 2 + 3,
+                }
+            }`,
             [2, 3, 4],
         ],
+
+        'does not find lines in empty var declarations': [
+            'var foo;',
+            [],
+        ],
+
+        'finds lines in empty let declarations': [
+            'let foo;',
+            [1],
+        ],
     };
 
     Object.keys(testTable).forEach(testcase => {
@@ -455,6 +465,109 @@ describe('Coverage.functionsForAST', function () {
                 { key: "(anonymous):2:0", line: 2, n_params: 0 }
             ],
         ],
+
+        'finds class methods': [
+            `class Foo {
+                bar() {}
+            }`,
+            [
+                { key: 'bar:2:0', line: 2, n_params: 0 },
+            ],
+        ],
+
+        'finds class property setters': [
+            `class Foo {
+                set bar(value) {}
+            }`,
+            [
+                { key: 'set bar:2:1', line: 2, n_params: 1 },
+            ],
+        ],
+
+        'finds class property getters': [
+            `class Foo {
+                get bar() {}
+            }`,
+            [
+                { key: 'get bar:2:0', line: 2, n_params: 0 },
+            ],
+        ],
+
+        'finds class constructors': [
+            `class Foo {
+                constructor(baz) {}
+            }`,
+            [
+                { key: 'Foo:2:1', line: 2, n_params: 1 },
+            ],
+        ],
+
+        'finds class expression methods': [
+            `void class {
+                baz() {}
+            }`,
+            [
+                { key: 'baz:2:0', line: 2, n_params: 0 },
+            ],
+        ],
+
+        'finds class expression property setters': [
+            `void class {
+                set baz(value) {}
+            }`,
+            [
+                { key: 'set baz:2:1', line: 2, n_params: 1 },
+            ],
+        ],
+
+        'finds class expression property getters': [
+            `void class {
+                get baz() {}
+            }`,
+            [
+                { key: 'get baz:2:0', line: 2, n_params: 0 },
+            ],
+        ],
+
+        'finds class expression constructors': [
+            `void class {
+                constructor(baz) {}
+            }`,
+            [
+                { key: '(anonymous):2:1', line: 2, n_params: 1 },
+            ],
+        ],
+
+        'finds functions inside labeled statement': [
+            `loop:
+            for (function () {}; ; ) {}`,
+            [
+                { key: '(anonymous):2:0', line: 2, n_params: 0 },
+            ],
+        ],
+
+        'finds functions inside switch expression': [
+            'switch (function () {}) {}',
+            [
+                { key: '(anonymous):1:0', line: 1, n_params: 0 },
+            ],
+        ],
+
+        'finds functions inside function default arguments': [
+            'function foo(bar=function () {}) {}',
+            [
+                { key: 'foo:1:1', line: 1, n_params: 1 },
+                { key: '(anonymous):1:0', line: 1, n_params: 0 },
+            ],
+        ],
+
+        'finds functions inside function expression default arguments': [
+            'void function foo(bar=function () {}) {}',
+            [
+                { key: 'foo:1:1', line: 1, n_params: 1 },
+                { key: '(anonymous):1:0', line: 1, n_params: 0 },
+            ],
+        ],
     };
 
     Object.keys(testTable).forEach(testcase => {
@@ -723,55 +836,21 @@ describe('Coverage', function () {
     });
 
     it('function key from function with name matches schema', function () {
+        let ast = Reflect.parse('function f(a, b) {}').body[0];
         let functionKeyForFunctionName =
-            Coverage._getFunctionKeyFromReflectedFunction({
-                id: {
-                    name: 'f'
-                },
-                loc: {
-                  start: {
-                      line: 1
-                  }
-                },
-                params: ['a', 'b']
-            });
+            Coverage._getFunctionKeyFromReflectedFunction(ast);
         expect(functionKeyForFunctionName).toEqual('f:1:2');
     });
 
     it('function key from function without name is anonymous', function () {
+        let ast = Reflect.parse('\nvoid function (a, b, c) {}').body[0].expression.argument;
         let functionKeyForAnonymousFunction =
-            Coverage._getFunctionKeyFromReflectedFunction({
-                id: null,
-                loc: {
-                  start: {
-                      line: 2
-                  }
-                },
-                params: ['a', 'b', 'c']
-            });
+            Coverage._getFunctionKeyFromReflectedFunction(ast);
         expect(functionKeyForAnonymousFunction).toEqual('(anonymous):2:3');
     });
 
     it('returns a function counter map for function keys', function () {
-        let ast = {
-            body: [{
-                type: 'FunctionDeclaration',
-                id: {
-                    name: 'name'
-                },
-                loc: {
-                  start: {
-                      line: 1
-                  }
-                },
-                params: [],
-                body: {
-                    type: 'BlockStatement',
-                    body: []
-                }
-            }]
-        };
-
+        let ast = Reflect.parse('function name() {}');
         let detectedFunctions = Coverage.functionsForAST(ast);
         let functionCounters =
             Coverage._functionsToFunctionCounters('script', detectedFunctions);
@@ -780,40 +859,7 @@ describe('Coverage', function () {
 
     it('reports an error when two indistinguishable functions are present', function () {
         spyOn(window, 'log');
-        let ast = {
-            body: [{
-                type: 'FunctionDeclaration',
-                id: {
-                    name: '(anonymous)'
-                },
-                loc: {
-                  start: {
-                      line: 1
-                  }
-                },
-                params: [],
-                body: {
-                    type: 'BlockStatement',
-                    body: []
-                }
-            }, {
-                type: 'FunctionDeclaration',
-                id: {
-                    name: '(anonymous)'
-                },
-                loc: {
-                  start: {
-                      line: 1
-                  }
-                },
-                params: [],
-                body: {
-                    type: 'BlockStatement',
-                    body: []
-                }
-            }]
-        };
-
+        let ast = Reflect.parse('() => {}; () => {}');
         let detectedFunctions = Coverage.functionsForAST(ast);
         Coverage._functionsToFunctionCounters('script', detectedFunctions);
 
@@ -927,25 +973,7 @@ describe('Coverage.incrementFunctionCounters', function () {
     });
 
     it('increments for function on earlier start line', function () {
-        let ast = {
-            body: [{
-                type: 'FunctionDeclaration',
-                id: {
-                    name: 'name'
-                },
-                loc: {
-                  start: {
-                      line: 1
-                  }
-                },
-                params: [],
-                body: {
-                    type: 'BlockStatement',
-                    body: []
-                }
-            }]
-        };
-
+        let ast = Reflect.parse('function name() {}');
         let detectedFunctions = Coverage.functionsForAST(ast);
         let knownFunctionsArray = Coverage._populateKnownFunctions(detectedFunctions, 3);
         let functionCounters = Coverage._functionsToFunctionCounters('script',
@@ -959,24 +987,7 @@ describe('Coverage.incrementFunctionCounters', function () {
     });
 
     it('throws an error on unexpected function', function () {
-        let ast = {
-            body: [{
-                type: 'FunctionDeclaration',
-                id: {
-                    name: 'name'
-                },
-                loc: {
-                  start: {
-                      line: 1
-                  }
-                },
-                params: [],
-                body: {
-                    type: 'BlockStatement',
-                    body: []
-                }
-            }]
-        };
+        let ast = Reflect.parse('function name() {}');
         let detectedFunctions = Coverage.functionsForAST(ast);
         let knownFunctionsArray = Coverage._populateKnownFunctions(detectedFunctions, 3);
         let functionCounters = Coverage._functionsToFunctionCounters('script',
@@ -1033,18 +1044,32 @@ describe('Coverage.incrementFunctionCounters', function () {
 
 describe('Coverage statistics container', function () {
     const MockFiles = {
-        'filename': "function f() {\n" +
-                    "    return 1;\n" +
-                    "}\n" +
-                    "if (f())\n" +
-                    "    f = 0;\n" +
-                    "\n",
-        'uncached': "function f() {\n" +
-                    "    return 1;\n" +
-                    "}\n"
+        'prefix/filename':
+            `function f() {
+                return 1;
+            }
+            if (f())
+                f = 0;
+            `,
+        'prefix/uncached':
+            `function f() {
+                return 1;
+            }
+            `,
+        'unprefixed':
+            `function f() {
+                return 1;
+            }
+            `,
+        'prefix/shebang':
+            `#!/usr/bin/env gjs
+            function f() {}
+            `,
     };
 
-    const MockFilenames = Object.keys(MockFiles).concat(['nonexistent']);
+    const MockFilenames = Object.keys(MockFiles).concat(['prefix/nonexistent']);
+
+    let container;
 
     beforeEach(function () {
         Coverage.getFileContents =
@@ -1053,25 +1078,38 @@ describe('Coverage statistics container', function () {
             jasmine.createSpy('getFileChecksum').and.returnValue('abcd');
         Coverage.getFileModificationTime =
             jasmine.createSpy('getFileModificationTime').and.returnValue([1, 2]);
+        container = new Coverage.CoverageStatisticsContainer(['prefix/']);
     });
 
     it('fetches valid statistics for file', function () {
-        let container = new Coverage.CoverageStatisticsContainer(MockFilenames);
-
-        let statistics = container.fetchStatistics('filename');
+        let statistics = container.fetchStatistics('prefix/filename');
         expect(statistics).toBeDefined();
 
         let files = container.getCoveredFiles();
-        expect(files).toEqual(['filename']);
+        expect(files).toEqual(['prefix/filename']);
     });
 
     it('throws for nonexisting file', function () {
-        let container = new Coverage.CoverageStatisticsContainer(MockFilenames);
-        expect(() => container.fetchStatistics('nonexistent')).toThrow();
+        expect(() => container.fetchStatistics('prefix/nonexistent')).toThrow();
+    });
+
+    it('handles a shebang on line 1', function () {
+        let statistics = container.fetchStatistics('prefix/shebang');
+        expect(statistics).toBeDefined();
+    });
+
+    it('ignores a file in angle brackets (our convention for programmatic scripts)', function () {
+        let statistics = container.fetchStatistics('<script>');
+        expect(statistics).not.toBeDefined();
+    });
+
+    it('ignores a file without the specified prefix', function () {
+        let statistics = container.fetchStatistics('unprefixed');
+        expect(statistics).not.toBeDefined();
     });
 
     const MockCache = '{ \
-        "filename": { \
+        "prefix/filename": { \
             "mtime": [1, 2], \
             "checksum": null, \
             "lines": [2, 4, 5], \
@@ -1099,22 +1137,22 @@ describe('Coverage statistics container', function () {
         });
 
         it('fetches counters from cache', function () {
-            container.fetchStatistics('filename');
+            container.fetchStatistics('prefix/filename');
             expect(Coverage._fetchCountersFromReflection).not.toHaveBeenCalled();
         });
 
         it('fetches counters from reflection if missed', function () {
-            container.fetchStatistics('uncached');
+            container.fetchStatistics('prefix/uncached');
             expect(Coverage._fetchCountersFromReflection).toHaveBeenCalled();
         });
 
         it('cache is not stale if all hit', function () {
-            container.fetchStatistics('filename');
+            container.fetchStatistics('prefix/filename');
             expect(container.staleCache()).toBeFalsy();
         });
 
         it('cache is stale if missed', function () {
-            container.fetchStatistics('uncached');
+            container.fetchStatistics('prefix/uncached');
             expect(container.staleCache()).toBeTruthy();
         });
     });
@@ -1125,10 +1163,10 @@ describe('Coverage statistics container', function () {
         beforeEach(function () {
             container = new Coverage.CoverageStatisticsContainer(MockFilenames,
                                                                  MockCache);
-            statistics = container.fetchStatistics('filename');
+            statistics = container.fetchStatistics('prefix/filename');
 
             containerWithNoCaching = new Coverage.CoverageStatisticsContainer(MockFilenames);
-            statisticsWithNoCaching = containerWithNoCaching.fetchStatistics('filename');
+            statisticsWithNoCaching = containerWithNoCaching.fetchStatistics('prefix/filename');
         });
 
         it('have same executable lines as reflection', function () {
