@@ -238,9 +238,6 @@ _Pragma("GCC diagnostic pop")
     case GJS_DEBUG_GERROR:
         prefix = "JS G ERR";
         break;
-    case GJS_DEBUG_PROXY:
-        prefix = "JS CPROXY";
-        break;
     default:
         prefix = "???";
         break;
